import * as logger from './log'
import { mqttHandler } from './MqttDispatcher'
import { AItem } from './AItem'
import { SwitchExport, ExportType } from './Export'
import { HasStateAndFeedbackTopicAndLabelAndRoom } from './AItem';

export class SimpleTopicSwitchItem extends AItem implements HasStateAndFeedbackTopicAndLabelAndRoom {
    private oldState: string|undefined
    private state: string
    private actionTopic: string
    private deviceFeedbackTopic: string
    private stateFeedbackTopic: string
    private stateTopic: string
    private type: string
    private partId: number

    getLabel(): string {
        return this.label
    }

    getRoom() : string {
        return this.room
    }

    getStateTopic() : string {
        return this.stateTopic
    }

    getStateFeedbackTopic() : string {
        return this.stateFeedbackTopic
    }

    getState() : string {
        return this.state
    }

    constructor(floor: string, room: string, item: string, label: string, actionTopic: string) {
        super(floor, room, item, label)
        this.stateTopic = `${this.topicFirstPart}/state`
        this.stateFeedbackTopic = `${this.topicFirstPart}/state/feedback`
        this.actionTopic = actionTopic
        this.subscribeTopics = [
            this.stateTopic
        ]
        this.state = 'OFF'
        this.oldState = undefined
    }

    exportItem() : ExportType|null {
        return SwitchExport(this.itemId, this.label, this.stateTopic, this.stateFeedbackTopic, this.type)
    }

    processMessage(topic: string, payload: string) : void {
        switch (topic) {
            case this.stateTopic:
                this.state = payload
                mqttHandler.send(this.stateFeedbackTopic, this.state)
                if (this.state != this.oldState) {                
                    if (this.state == 'ON') {
                        mqttHandler.send(this.actionTopic, 'ON')
                    } else {
                        mqttHandler.send(this.actionTopic, 'OFF')
                    }
                    this.oldState = this.state
                }
                break
        }
        this.emit('somethingChanged')
    }
}
